@@ -117,6 +117,14 @@ PythonImmutableTypes = typing.Union[
 
 @dataclasses.dataclass  # type: ignore
 class BaseField:
+
+    __slots__ = (
+        "name",
+        "type",
+        "default",
+        "parent",
+    )
+
     name: str
     type: typing.Any  # store the python primitive type
     default: typing.Any
@@ -125,7 +133,7 @@ class BaseField:
     model_metadata: typing.Optional[utils.SchemaMetadata] = None
 
     def __post_init__(self) -> None:
-        self.model_metadata = self.model_metadata or utils.SchemaMetadata()
+        self.model_metadata = self.model_metadata or utils.SchemaMetadata()  # type: ignore
 
     @property
     def avro_type(self) -> typing.Union[str, typing.Dict]:
@@ -215,6 +223,18 @@ class BaseField:
     def fake(self) -> typing.Any:
         return None
 
+    def exist_type(self) -> int:
+        # filter by the same field types
+        same_types = [
+            field.type
+            for field in self.parent.user_defined_types
+            if field.type == self.type and field.name != self.name
+        ]
+
+        # If length > 0, means that it is the first appearance
+        # of this type, otherwise exist already.
+        return len(same_types)
+
 
 class ImmutableField(BaseField):
     def get_avro_type(self) -> PythonImmutableTypes:
@@ -483,7 +503,6 @@ class UnionField(BaseField):
         return self.default
 
     def fake(self) -> typing.Any:
-        self.unions = self.generate_unions_type()
         # get a random internal field and return a fake value
         field = random.choice(self.internal_fields)
         return field.fake()
@@ -528,15 +547,24 @@ class EnumField(BaseField):
     def get_symbols(self) -> typing.List[str]:
         return [member.value for member in self.type if member.name != "Meta"]
 
-    def get_avro_type(self) -> JsonDict:
-        avro_type = {
-            "type": ENUM,
-            "name": self.get_singular_name(self.name),
-            "symbols": self.get_symbols(),
-            **self._get_meta_class_attributes(),
-        }
+    def get_avro_type(self) -> typing.Union[str, JsonDict]:
+        metadata = self._get_meta_class_attributes()
+        name = self.type.__name__
 
-        return avro_type
+        if not self.exist_type():
+            user_defined_type = utils.UserDefinedType(name=name, type=self.type)
+            self.parent.user_defined_types.add(user_defined_type)
+            return {
+                "type": ENUM,
+                "name": name,
+                "symbols": self.get_symbols(),
+                **metadata,
+            }
+        else:
+            namespace = metadata.get("namespace")
+            if namespace is None:
+                raise NameSpaceRequiredException(field_type=self.type, field_name=name)
+            return f"{namespace}.{name}"
 
     def get_default_value(self) -> typing.Union[str, dataclasses._MISSING_TYPE, None]:
 
@@ -687,7 +715,7 @@ class TimeMicroField(LogicalTypeField):
             time (datetime.time)
 
         Returns:
-            float
+            int
         """
         hour, minutes, seconds, microseconds = (
             time.hour,
@@ -696,7 +724,7 @@ class TimeMicroField(LogicalTypeField):
             time.microsecond,
         )
 
-        return float((((hour * 60 + minutes) * 60 + seconds) * 1000000) + microseconds)
+        return int((((hour * 60 + minutes) * 60 + seconds) * 1000000) + microseconds)
 
     def fake(self) -> datetime.time:
         datetime_object: datetime.datetime = fake.date_time(tzinfo=utc)
@@ -736,7 +764,7 @@ class DatetimeField(LogicalTypeField):
         else:
             ts = (date_time - utils.epoch_naive).total_seconds()
 
-        return ts * 1000
+        return int(ts * 1000)
 
     def fake(self) -> datetime.datetime:
         return fake.date_time(tzinfo=utc)
@@ -774,7 +802,7 @@ class DatetimeMicroField(LogicalTypeField):
         else:
             ts = (date_time - utils.epoch_naive).total_seconds()
 
-        return ts * 1000000
+        return int(ts * 1000000)
 
     def fake(self) -> datetime.datetime:
         datetime_object: datetime.datetime = fake.date_time(tzinfo=utc)
@@ -803,7 +831,7 @@ class UUIDField(LogicalTypeField):
 
 @dataclasses.dataclass
 class RecordField(BaseField):
-    def get_avro_type(self) -> typing.Union[typing.List, typing.Dict]:
+    def get_avro_type(self) -> typing.Union[str, typing.List, typing.Dict]:
         meta = getattr(self.type, "Meta", type)
         metadata = utils.SchemaMetadata.create(meta)
 
@@ -833,18 +861,6 @@ class RecordField(BaseField):
     def fake(self) -> typing.Any:
         return self.type.fake()
 
-    def exist_type(self) -> int:
-        # filter by the same field types
-        same_types = [
-            field.type
-            for field in self.parent.user_defined_types
-            if field.type == self.type and field.name != self.name
-        ]
-
-        # If length > 0, means that it is the first appearance
-        # of this type, otherwise exist already.
-        return len(same_types)
-
 
 @dataclasses.dataclass
 class DecimalField(BaseField):
@@ -902,6 +918,7 @@ class DecimalField(BaseField):
 
     def get_default_value(self) -> typing.Union[str, dataclasses._MISSING_TYPE, None]:
         default = self.default
+
         if isinstance(default, types.Decimal):
             default = default.default
 
@@ -909,6 +926,7 @@ class DecimalField(BaseField):
             return dataclasses.MISSING
         if default is None:
             return None
+
         return serialization.decimal_to_str(default, self.precision, self.scale)
 
     def fake(self) -> decimal.Decimal:
@@ -987,7 +1005,7 @@ def field_factory(
     default: typing.Any = dataclasses.MISSING,
     default_factory: typing.Any = dataclasses.MISSING,
     metadata: typing.Optional[typing.Mapping] = None,
-    model_metadata: utils.SchemaMetadata = None,
+    model_metadata: typing.Optional[utils.SchemaMetadata] = None,
 ) -> FieldType:
     if metadata is None:
         metadata = {}
@@ -1070,6 +1088,20 @@ def field_factory(
             model_metadata=model_metadata,
             parent=parent,
         )
+    elif types.UnionType is not None and isinstance(native_type, types.UnionType):
+        # we need to check whether types.UnionType because it works only in
+        # python 3.9 or importing __future__ in previous python versions
+        # cases when a container is used, for example `typing.List[int] | str` in python is
+        # translated to typing.Union[typing.List[int], str] so it won't reach this point
+        return UnionField(
+            name=name,
+            type=native_type,
+            default=default,
+            metadata=metadata,
+            default_factory=default_factory,
+            model_metadata=model_metadata,
+            parent=parent,
+        )
     elif inspect.isclass(native_type) and issubclass(native_type, schema_generator.AvroModel):
         return RecordField(
             name=name,
