@@ -13,6 +13,17 @@ from . import consts
 faker = Faker()
 
 
+class Color(enum.Enum):
+    BLUE = "Blue"
+    GREEN = "Green"
+    YELLOW = "Yellow"
+
+    class Meta:
+        aliases = ["one", "two"]
+        doc = "colors"
+        namespace = "some.name.space"
+
+
 def test_invalid_type_container_field():
     python_type = typing.Set
     name = "test_field"
@@ -232,26 +243,23 @@ def test_mapping_logical_type(mapping, python_primitive_type, python_type_str, v
     assert expected == field.to_dict()
 
 
-@pytest.mark.parametrize("primitive_types, avro_types, default", consts.UNION_PRIMITIVE_ELEMENTS)
-def test_union_type(primitive_types, avro_types, default):
+@pytest.mark.parametrize("union, avro_types", consts.UNION_PRIMITIVE_ELEMENTS)
+def test_union_type(union, avro_types) -> None:
     name = "an_union_field"
-    python_type = typing.Union[primitive_types]
-    field = fields.AvroField(name, python_type)
-
+    field = fields.AvroField(name, union)
     expected = {"name": name, "type": [*avro_types]}
 
     assert expected == field.to_dict()
 
 
 # Tests to make sure defaults work, and that defaults are sorted to the beginning of the union
-@pytest.mark.parametrize("primitive_types, avro_types, default", consts.UNION_PRIMITIVE_ELEMENTS_DEFAULTS)
-def test_union_type_with_default(primitive_types, avro_types, default):
+@pytest.mark.parametrize("union, avro_types, default", consts.UNION_PRIMITIVE_ELEMENTS_DEFAULTS)
+def test_union_type_with_default(union, avro_types, default) -> None:
     name = "an_union_field"
-    python_type = typing.Union[primitive_types]
-    field = fields.AvroField(name, python_type, default=default)
+    field = fields.AvroField(name, union, default=default)
 
     if isinstance(default, datetime.datetime):
-        default = (default - datetime.datetime(1970, 1, 1)).total_seconds() * 1000
+        default = int((default - datetime.datetime(1970, 1, 1)).total_seconds() * 1000)
     elif isinstance(default, bytes):
         default = default.decode()
 
@@ -260,22 +268,20 @@ def test_union_type_with_default(primitive_types, avro_types, default):
     assert expected == field.to_dict()
 
 
-@pytest.mark.parametrize("complex_type, avro_types", consts.UNION_WITH_ARRAY)
-def test_union_with_arrays(complex_type, avro_types):
+@pytest.mark.parametrize("union, avro_types", consts.UNION_WITH_ARRAY)
+def test_union_with_arrays(union, avro_types) -> None:
     name = "an_union_field"
-    python_type = typing.Union[complex_type]
-    field = fields.AvroField(name, python_type)
+    field = fields.AvroField(name, union)
 
     expected = {"name": name, "type": [{"type": "array", "name": name, "items": avro_types[0]}, avro_types[1]]}
 
     assert expected == field.to_dict()
 
 
-@pytest.mark.parametrize("complex_type, avro_types", consts.UNION_WITH_MAP)
-def test_union_with_maps(complex_type, avro_types):
+@pytest.mark.parametrize("union, avro_types", consts.UNION_WITH_MAP)
+def test_union_with_maps(union, avro_types) -> None:
     name = "an_union_field"
-    python_type = typing.Union[complex_type]
-    field = fields.AvroField(name, python_type)
+    field = fields.AvroField(name, union)
 
     expected = {"name": name, "type": [{"type": "map", "name": name, "values": avro_types[0]}, avro_types[1]]}
 
@@ -283,7 +289,7 @@ def test_union_with_maps(complex_type, avro_types):
 
 
 @pytest.mark.parametrize("complex_type, avro_type", consts.OPTIONAL_UNION_COMPLEX_TYPES)
-def test_union_as_optional_with_complex_types(complex_type, avro_type):
+def test_union_as_optional_with_complex_types(complex_type, avro_type) -> None:
     """
     Test cases when typing.Optional is used.
     The result of typing.Optional[Any] is typing.Union[Any, NoneType]
@@ -300,7 +306,7 @@ def test_union_as_optional_with_complex_types(complex_type, avro_type):
 
 
 @pytest.mark.parametrize("primitive_type, avro_type", consts.PRIMITIVE_TYPES)
-def test_union_as_optional_with_primitives(primitive_type, avro_type):
+def test_union_as_optional_with_primitives(primitive_type, avro_type) -> None:
     """
     Test cases when typing.Optional is used.
     The result of typing.Optional[Any] is typing.Union[Any, NoneType]
@@ -447,6 +453,7 @@ def test_enum_type():
     name = "an_enum_field"
     namespace = "my_enum"
     aliases = ["enum", "first enum"]
+    parent = AvroModel()
 
     class CardType(enum.Enum):
         SPADES = "SPADES"
@@ -459,14 +466,14 @@ def test_enum_type():
             aliases = ["enum", "first enum"]
 
     python_type = CardType
-    field = fields.AvroField(name, python_type, default=CardType.CLUBS)
+    field = fields.AvroField(name, python_type, default=CardType.CLUBS, parent=parent)
     symbols = ["SPADES", "HEARTS", "DIAMONDS", "CLUBS"]
 
     expected = {
         "name": name,
         "type": {
             "type": "enum",
-            "name": name,
+            "name": "CardType",
             "symbols": symbols,
             "namespace": namespace,
             "aliases": aliases,
@@ -486,9 +493,9 @@ def test_enum_type():
             namespace = "my_enum"
 
     python_type = CardType
-    field = fields.AvroField(name, python_type)
+    field = fields.AvroField(name, python_type, parent=parent)
 
-    expected = {"name": name, "type": {"type": "enum", "name": name, "symbols": symbols, "namespace": namespace}}
+    expected = {"name": name, "type": {"type": "enum", "name": "CardType", "symbols": symbols, "namespace": namespace}}
 
     assert expected == field.to_dict()
 
@@ -499,13 +506,13 @@ def test_enum_type():
         CLUBS = "CLUBS"
 
     python_type = CardType
-    field = fields.AvroField(name, python_type, default=None)
+    field = fields.AvroField(name, python_type, default=None, parent=parent)
 
     expected = {
         "name": name,
         "type": {
             "type": "enum",
-            "name": name,
+            "name": "CardType",
             "symbols": symbols,
         },
         "default": None,
@@ -513,6 +520,25 @@ def test_enum_type():
 
     assert expected == field.to_dict()
 
+    python_type = typing.Optional[CardType]
+    parent.user_defined_types = set()
+    field = fields.AvroField(name, python_type, default=None, parent=parent)
+
+    expected = {
+        "name": name,
+        "type": [
+            "null",
+            {
+                "type": "enum",
+                "name": "CardType",
+                "symbols": symbols,
+            },
+        ],
+        "default": None,
+    }
+
+    assert expected == field.to_dict()
+
     with pytest.raises(AssertionError):
 
         class CardType(enum.Enum):
@@ -525,24 +551,14 @@ def test_enum_type():
             SOMETHING = "SOMETHING"
 
         python_type = CardType
-        field = fields.AvroField(name, python_type, default=RandomType.SOMETHING)
-
+        field = fields.AvroField(name, python_type, default=RandomType.SOMETHING, parent=parent)
         field.to_dict()
 
 
-class Color(enum.Enum):
-    BLUE = "Blue"
-    GREEN = "Green"
-    YELLOW = "Yellow"
-
-    class Meta:
-        aliases = ["one", "two"]
-        doc = "colors"
-        namespace = "some.name.space"
-
-
 def test_enum_field():
-    enum_field = fields.AvroField("field_name", Color, default=Color.BLUE, metadata={"key": "value"})
+    enum_field = fields.AvroField(
+        "field_name", Color, default=Color.BLUE, metadata={"key": "value"}, parent=AvroModel()
+    )
 
     assert enum_field.get_symbols() == ["Blue", "Green", "Yellow"]
     assert enum_field._get_meta_class_attributes() == {
@@ -552,7 +568,7 @@ def test_enum_field():
     }
     assert enum_field.get_avro_type() == {
         "type": "enum",
-        "name": "field_name",
+        "name": "Color",
         "symbols": ["Blue", "Green", "Yellow"],
         "aliases": ["one", "two"],
         "doc": "colors",
