@@ -49,7 +49,7 @@ def test_invalid_default_values(primitive_type, invalid_default):
 
 
 def test_invalid_type():
-    msg = f"Type {typing.Any} is unknown. Please check the valid types at https://marcosschroh.github.io/dataclasses-avroschema/fields_specification/#avro-field-and-python-types-summary"
+    msg = f"Type {typing.Any} is unknown. Please check the valid types at https://marcosschroh.github.io/dataclasses-avroschema/fields_specification/#avro-field-and-python-types-summary"  # noqa
     with pytest.raises(ValueError, match=msg):
         name = "a_field"
         fields.AvroField(name, typing.Any)
